@@ -1,7 +1,7 @@
 /* config-bot.h */
 /* modify settings or make new ones based on what autoconf tells us. */
 
-/* Copyright (C) 1989-2009 Free Software Foundation, Inc.
+/* Copyright (C) 1989-2021 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -36,17 +36,6 @@
 #  define HAVE_BSD_PGRP
 #endif
 
-/* Try this without testing __STDC__ for the time being. */
-#if defined (HAVE_STDARG_H)
-#  define PREFER_STDARG
-#  define USE_VARARGS
-#else
-#  if defined (HAVE_VARARGS_H)
-#    define PREFER_VARARGS
-#    define USE_VARARGS
-#  endif
-#endif
-
 #if defined (HAVE_SYS_SOCKET_H) && defined (HAVE_GETPEERNAME) && defined (HAVE_NETINET_IN_H)
 #  define HAVE_NETWORK
 #endif
@@ -99,7 +88,7 @@
 #endif
 
 #if !HAVE_MKDTEMP
-#  undef USE_MKDTMP
+#  undef USE_MKDTEMP
 #endif
 
 /* If the shell is called by this name, it will become restricted. */
