@@ -1,7 +1,7 @@
 /* true and false builtins */
 
 /*
-   Copyright (C) 1999-2009 Free Software Foundation, Inc.
+   Copyright (C) 1999-2009,2022 Free Software Foundation, Inc.
 
    This file is part of GNU Bash.
    Bash is free software: you can redistribute it and/or modify
@@ -26,15 +26,13 @@
 #include "common.h"
 
 int
-true_builtin (list)
-     WORD_LIST *list;
+true_builtin (WORD_LIST *list)
 {
   return EXECUTION_SUCCESS;
 }
 
 int
-false_builtin (list)
-     WORD_LIST *list;
+false_builtin (WORD_LIST *list)
 {
   return EXECUTION_FAILURE;
 }
