@@ -1,24 +1,23 @@
 /* emacs_keymap.c -- the keymap for emacs_mode in readline (). */
 
-/* Copyright (C) 1987, 1989, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2017 Free Software Foundation, Inc.
 
-   This file is part of the GNU Readline Library, a library for
-   reading lines of text with interactive input and history editing.
+   This file is part of the GNU Readline Library (Readline), a library
+   for reading lines of text with interactive input and history editing.      
 
-   The GNU Readline Library is free software; you can redistribute it
-   and/or modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2, or
+   Readline is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   The GNU Readline Library is distributed in the hope that it will be
-   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   Readline is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   The GNU General Public License is often shipped with GNU software, and
-   is generally kept in a file called COPYING or LICENSE.  If you do not
-   have a copy of the license, write to the Free Software Foundation,
-   59 Temple Place, Suite 330, Boston, MA 02111 USA. */
+   You should have received a copy of the GNU General Public License
+   along with Readline.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #if !defined (BUFSIZ)
 #include <stdio.h>
@@ -35,11 +34,11 @@ KEYMAP_ENTRY_ARRAY emacs_standard_keymap = {
   /* Control keys. */
   { ISFUNC, rl_set_mark },			/* Control-@ */
   { ISFUNC, rl_beg_of_line },			/* Control-a */
-  { ISFUNC, rl_backward },			/* Control-b */
+  { ISFUNC, rl_backward_char },			/* Control-b */
   { ISFUNC, (rl_command_func_t *)0x0 },		/* Control-c */
   { ISFUNC, rl_delete },			/* Control-d */
   { ISFUNC, rl_end_of_line },			/* Control-e */
-  { ISFUNC, rl_forward },			/* Control-f */
+  { ISFUNC, rl_forward_char },			/* Control-f */
   { ISFUNC, rl_abort },				/* Control-g */
   { ISFUNC, rl_rubout },			/* Control-h */
   { ISFUNC, rl_complete },			/* Control-i */
@@ -48,7 +47,7 @@ KEYMAP_ENTRY_ARRAY emacs_standard_keymap = {
   { ISFUNC, rl_clear_screen },			/* Control-l */
   { ISFUNC, rl_newline },			/* Control-m */
   { ISFUNC, rl_get_next_history },		/* Control-n */
-  { ISFUNC, (rl_command_func_t *)0x0 },		/* Control-o */
+  { ISFUNC, rl_operate_and_get_next },		/* Control-o */
   { ISFUNC, rl_get_previous_history },		/* Control-p */
   { ISFUNC, rl_quoted_insert },			/* Control-q */
   { ISFUNC, rl_reverse_search_history },	/* Control-r */
@@ -328,7 +327,7 @@ KEYMAP_ENTRY_ARRAY emacs_meta_keymap = {
   { ISFUNC, rl_tab_insert },		/* Meta-Control-i */
   { ISFUNC, rl_vi_editing_mode },	/* Meta-Control-j */
   { ISFUNC, (rl_command_func_t *)0x0 },	/* Meta-Control-k */
-  { ISFUNC, (rl_command_func_t *)0x0 },	/* Meta-Control-l */
+  { ISFUNC, rl_clear_display },		/* Meta-Control-l */
   { ISFUNC, rl_vi_editing_mode }, 	/* Meta-Control-m */
   { ISFUNC, (rl_command_func_t *)0x0 },	/* Meta-Control-n */
   { ISFUNC, (rl_command_func_t *)0x0 },	/* Meta-Control-o */
@@ -449,7 +448,7 @@ KEYMAP_ENTRY_ARRAY emacs_meta_keymap = {
   { ISFUNC, rl_upcase_word },		/* Meta-u */
   { ISFUNC, (rl_command_func_t *)0x0 },	/* Meta-v */
   { ISFUNC, (rl_command_func_t *)0x0 },	/* Meta-w */
-  { ISFUNC, (rl_command_func_t *)0x0 },	/* Meta-x */
+  { ISFUNC, rl_execute_named_command },	/* Meta-x */
   { ISFUNC, rl_yank_pop },		/* Meta-y */
   { ISFUNC, (rl_command_func_t *)0x0 },	/* Meta-z */
 
