@@ -2,7 +2,7 @@
  * rename - rename a file
  */
 
-/* Copyright (C) 1999 Free Software Foundation, Inc.
+/* Copyright (C) 1999, 2022 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -39,8 +39,7 @@ extern int errno;
 #endif
 
 int
-rename (from, to)
-     const char *from, *to;
+rename (const char *from, const char *to)
 {
   struct stat fb, tb;
 
