@@ -1,6 +1,6 @@
 /* ufuncs - sleep and alarm functions that understand fractional values */
 
-/* Copyright (C) 2008,2009 Free Software Foundation, Inc.
+/* Copyright (C) 2008,2009-2020,2022 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -22,16 +22,7 @@
 
 #include "bashtypes.h"
 
-#if defined (TIME_WITH_SYS_TIME)
-#  include <sys/time.h>
-#  include <time.h>
-#else
-#  if defined (HAVE_SYS_TIME_H)
-#    include <sys/time.h>
-#  else
-#    include <time.h>
-#  endif
-#endif
+#include "posixtime.h"
 
 #if defined (HAVE_UNISTD_H)
 #include <unistd.h>
@@ -42,19 +33,21 @@
 extern int errno;
 #endif /* !errno */
 
+#include "quit.h"
+
 #if defined (HAVE_SELECT)
 #  include "posixselect.h"
-#  include "quit.h"
 #  include "trap.h"
 #  include "stat-time.h"
 #endif
 
+#include "error.h"
+
 /* A version of `alarm' using setitimer if it's available. */
 
 #if defined (HAVE_SETITIMER)
 unsigned int
-falarm(secs, usecs)
-     unsigned int secs, usecs;
+falarm (unsigned int secs, unsigned int usecs)
 {
   struct itimerval it, oit;
 
@@ -74,8 +67,7 @@ falarm(secs, usecs)
 }
 #else
 int
-falarm (secs, usecs)
-     unsigned int secs, usecs;
+falarm (unsigned int secs, unsigned int usecs)
 {
   if (secs == 0 && usecs == 0)
     return (alarm (0));
@@ -92,58 +84,93 @@ falarm (secs, usecs)
 /* A version of sleep using fractional seconds and select.  I'd like to use
    `usleep', but it's already taken */
 
+#if defined (HAVE_NANOSLEEP)
+static int
+nsleep (unsigned int sec, unsigned int usec)
+{
+  int r;
+  struct timespec req, rem;
+
+  req.tv_sec = sec;
+  req.tv_nsec = usec * 1000;
+
+  for (;;)
+    {
+      QUIT;
+      r = nanosleep (&req, &rem);
+      if (r == 0 || errno != EINTR)
+	return r;
+      req = rem;
+    }
+  return r;
+}	
+#endif
+
 #if defined (HAVE_TIMEVAL) && (defined (HAVE_SELECT) || defined (HAVE_PSELECT))
-int
-fsleep(sec, usec)
-     unsigned int sec, usec;
+static int
+ssleep (unsigned int sec, unsigned int usec)
 {
   int e, r;
-  sigset_t blocked_sigs, prevmask;
-#if defined (HAVE_PSELECT)
+  sigset_t blocked_sigs;
+#  if defined (HAVE_PSELECT)
   struct timespec ts;
-#else
+#  else
+  sigset_t prevmask;
   struct timeval tv;
-#endif
+#  endif
 
   sigemptyset (&blocked_sigs);
 #  if defined (SIGCHLD)
   sigaddset (&blocked_sigs, SIGCHLD);
 #  endif
 
-#if defined (HAVE_PSELECT)
+#  if defined (HAVE_PSELECT)
   ts.tv_sec = sec;
   ts.tv_nsec = usec * 1000;
-#else
+#  else
   sigemptyset (&prevmask);
   tv.tv_sec = sec;
   tv.tv_usec = usec;
-#endif /* !HAVE_PSELECT */
+#  endif /* !HAVE_PSELECT */
 
   do
     {
-#if defined (HAVE_PSELECT)
+#  if defined (HAVE_PSELECT)
       r = pselect(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, &ts, &blocked_sigs);
-#else
+#  else
       sigprocmask (SIG_SETMASK, &blocked_sigs, &prevmask);
       r = select(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, &tv);
       sigprocmask (SIG_SETMASK, &prevmask, NULL);
-#endif
+#  endif
       e = errno;
       if (r < 0 && errno == EINTR)
-	QUIT;		/* just signals, no traps */
+	return -1;		/* caller will handle XXX - QUIT;? */
       errno = e;
     }
   while (r < 0 && errno == EINTR);
 
   return r;
 }
-#else /* !HAVE_TIMEVAL || !HAVE_SELECT */
-int
-fsleep(sec, usec)
-     long sec, usec;
+#endif
+
+#if !defined (HAVE_SELECT)
+static int
+ancientsleep(unsigned int sec, unsigned int usec)
 {
   if (usec >= 500000)	/* round */
    sec++;
   return (sleep(sec));
 }
+#endif
+
+int
+fsleep(unsigned int sec, unsigned int usec)
+{
+#if defined (HAVE_NANOSLEEP)
+  return (nsleep (sec, usec));
+#elif defined (HAVE_TIMEVAL) && (defined (HAVE_SELECT) || defined (HAVE_PSELECT))
+  return (ssleep (sec, usec));
+#else /* !HAVE_TIMEVAL || !HAVE_SELECT */
+  return (ancientsleep (sec, usec));
 #endif /* !HAVE_TIMEVAL || !HAVE_SELECT */
+}
